import breeze.linalg.{DenseVector => BDV}
import breeze.linalg.{DenseMatrix => BDM}
import breeze.optimize.DiffFunction
import breeze.linalg._


object ErrorMetric{
   final val RMSE:Int = 1
   final val MAD:Int  = 2
  
}

/**
 * @author Nodalpoint
 * 
 * Abstract definition of an error (or loss) objective function for the case of sum-of-squares.
 * We extend Breeze's DiffFunction so that we can use Breeze optimization algorithms for comparison!
 * 
 * getDim(): Int
 *  Returns the dimensionality of the objective function (the tunable parameters
 *  of the model)
 *  
 * calculate(weights: BDV[Double]): (Double, BDV[Double])
 *  Provided a set of adjustable weights it returns the function value and the gradient vector
 *  
 * hessian(weights: BDV[Double]): BDM[Double]  
 *  Provided a set of adjustable weights it returns the second order derivatives (Hessian matrix)
 *   
 */
trait SumOfSquaresFunction extends  DiffFunction[BDV[Double]] {
  def getDim():Int
  def calculate(weights: BDV[Double]): (Double, BDV[Double])
  def hessian(weights: BDV[Double]): BDM[Double]  
 }